package org.gfxtest.framework;

import java.awt.image.BufferedImage;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * This class is used to store actual configuration of graphics test framework.
 * Configuration can be changed via command line switches.
 *
 * @author Pavel Tisnovsky
 */
public class Configuration
{
    /**
     * Instance of logger class.
     */
    private Log log;

    /**
     * Path to generated output files.
     */
    private File outputPath;

    /**
     * Width (measured in pixels) of the generated images.
     */
    private int imageWidth;

    /**
     * Height (measured in pixels) of the generated images.
     */
    private int imageHeight;

    /**
     * Image type. Currently are supported these image types:
     * BufferedImage.TYPE_INT_RGB, BufferedImage.TYPE_BYTE_GRAY and
     * BufferedImage.TYPE_BYTE_INDEXED.
     */
    private int imageType;

    public Configuration(String[] args, Log log) throws ConfigurationException
    {
        this.log = log;
        log.logBegin("read configuration");
        readCommandLineParameters(args);
        log.logEnd("read configuration");
        printParameters();
    }

    private void readCommandLineParameters(String[] args) throws ConfigurationException
    {
        Map<String, String> options = resolveAllOptions(args);
        this.imageWidth = getIntegerParameter(options, "w");
        this.imageHeight = getIntegerParameter(options, "h");
        this.imageType = getImageType(options);
        this.outputPath = new File(getStringParameter(options, "o"));
    }

    private int getImageType(Map<String, String> options) throws ConfigurationException
    {
        if (!options.containsKey("t"))
        {
            throw new ParameterNotFoundException("t");
        }
        String imageType = options.get("t");
        if ("rgb".equals(imageType))
        {
            return BufferedImage.TYPE_INT_RGB;
        }
        else if ("gray".equals(imageType))
        {
            return  BufferedImage.TYPE_BYTE_GRAY;
        }
        else if ("bw".equals(imageType) || ("bit".equals(imageType)))
        {
            return BufferedImage.TYPE_BYTE_INDEXED;
        }
        else
        {
            throw new InvalidParameterValueException("t", imageType); 
        }
    }

    private int getIntegerParameter(Map<String, String> options, String parameterName) throws ConfigurationException
    {
        if (!options.containsKey(parameterName))
        {
            throw new ParameterNotFoundException(parameterName);
        }
        String parameterValue = options.get(parameterName);
        try
        {
            return Integer.parseInt(parameterValue);
        }
        catch (NumberFormatException e)
        {
            throw new InvalidParameterValueException(parameterName, parameterValue);
        }
    }

    private String getStringParameter(Map<String, String> options, String parameterName) throws ConfigurationException
    {
        if (!options.containsKey(parameterName))
        {
            throw new ParameterNotFoundException(parameterName);
        }
        return options.get(parameterName);
    }

    private Map<String, String> resolveAllOptions(String[] args)
    {
        Map<String, String> options = new HashMap<String, String>();
        for (String arg : args)
        {
            String[] splittedArg = arg.split("=");
            if (splittedArg.length >= 1 && splittedArg[0].length() >= 2)
            {
                log.log("found option " + arg);
                String optionName = splittedArg[0].substring(1);
                String optionValue = splittedArg.length == 2 ? splittedArg[1] : "true";
                options.put(optionName, optionValue);
            }
            else
            {
                log.logError("unknown option " + arg);
            }
        }
        return options;
    }

    private void printParameters()
    {
        log.logSet("output path", this.outputPath);
        log.logSet("image width", this.imageWidth);
        log.logSet("image height", this.imageHeight);
        log.logSet("image type", this.imageType);
    }

    public File getOutputPath()
    {
        return outputPath;
    }

    public int getImageWidth()
    {
        return imageWidth;
    }

    public int getImageHeight()
    {
        return imageHeight;
    }

    public int getImageType()
    {
        return imageType;
    }

}
