package org.gfxtest.framework;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public abstract class GfxTest
{
    protected Log log = null;
    protected String suiteName = null;
    protected static final int CROSS_SIZE = 20;

    /**
     * Write test suite duration to the log file.
     *
     * @param t1 time when the test suite has been started
     * @param t2 time when the test suite was completed
     */
    private void printDuration(long t1, long t2)
    {
        log.logSet("duration", (t2-t1));
    }

    /**
     * Construct test suite name using full class name.
     */
    private void fillSuiteName()
    {
        String className = this.getClass().getName();
        this.suiteName = className.substring(1 + className.lastIndexOf('.'));
    }

    protected void runTestSuite(String[] args)
    {
        long t1 = System.currentTimeMillis();
        log = new Log(this.getClass().getName(), true);
        log.logBegin("testsuite");
        fillSuiteName();
        Configuration configuration;
        try
        {
            configuration = new Configuration(args, log);
            runAllTests(configuration);
        }
        catch (ConfigurationException e)
        {
            e.printStackTrace();
        }
        finally
        {
            long t2 = System.currentTimeMillis();
            log.logEnd("testsuite");
            printDuration(t1, t2);
        }
    }

    /**
     * Invoke all methods which names start with "test" prefix. These methods
     * should accept one parameter of type Buffered image.
     *
     * @param configuration
     * @throws IOException
     * @throws IllegalArgumentException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    private void runAllTests(Configuration configuration)
    {
        log.logBegin("common tests");
        Method[] methods = getClass().getDeclaredMethods();
        for (Method method : methods)
        {
            tryToInvokeTestMethod(configuration, method);
        }
        log.logEnd("common tests");
        log.logBegin("other tests");
        runOtherTests(configuration);
        log.logEnd("other tests");
    }

    /**
     * Method which can be overridden by inherited classes.
     *
     * @param configuration configuration of current test case.
     */
    protected void runOtherTests(Configuration configuration)
    {
    }

    private void tryToInvokeTestMethod(Configuration configuration, Method method)
    {
        String methodName = method.getName();
        if (method.getName().startsWith("test"))
        {
            log.logBegin(methodName);
            TestImage image = new TestImage(configuration);
            Graphics2D gc = (Graphics2D) image.getGraphics();
            TestResult result = null;
            try
            {
                result = (TestResult) method.invoke(this, new Object[] { image, gc });
            }
            catch (Exception e)
            {
                e.printStackTrace();
                result = TestResult.ERROR;
            }
            finally
            {
                gc.dispose();
                logTestResult(result);
                try
                {
                    image.writeImage(configuration, suiteName, methodName);
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }
                log.logEnd(methodName);
            }
        }
    }

    /**
     * Write test result to log file.
     *
     * @param result status of the last invoked test
     */
    protected void logTestResult(TestResult result)
    {
        String resultText = "";
        switch (result)
        {
        case PASSED:
            resultText = "$GREEN$PASSED";
            break;
        case FAILED:
            resultText = "$RED$FAILED";
            break;
        case ERROR:
            resultText = "$BLUE$ERROR";
            break;
        }
        log.logSet("test result", resultText);
    }

    /**
     * 
     * @param graphics graphics context for the image created by test case
     * @param x x-coordinate of the cross center
     * @param y y-coordinate of the cross center
     * @param size size of cross (measured in pixels)
     */
    protected void drawCross(Graphics2D graphics, int x, int y, int size)
    {
        graphics.setStroke(new BasicStroke(1.0f));
        graphics.setColor(Color.RED);
        graphics.drawLine(x, y - size, x, y + size);
        graphics.drawLine(x - size, y, x + size, y);
    }

}
