package org.gfxtest.framework;

import java.util.Date;
import java.util.HashMap;

/**
 * Simple logger that writes everything out to standard output and can use color
 * codes for ANSI-compatible terminals.
 *
 * @author Pavel Tisnovsky
 */
public class Log
{
    static HashMap<String, String> colors = new HashMap<String, String>(10);

    static
    {
        colors.put("RED", "31");
        colors.put("GREEN", "32");
        colors.put("ORANGE", "33");
        colors.put("BLUE", "34");
        colors.put("VIOLET", "35");
        colors.put("MAGENTA", "35");
        colors.put("LIGHT_BLUE", "36");
        colors.put("GRAY", "38");
        colors.put("GREY", "38");
    }

    /**
     * Current indentation level. Increased in logBegin() and decreased in
     * logEnd() methods.
     */
    private int indent = 0;

    /**
     * Indentation level rate.
     */
    private int indentDelta = 2;

    /**
     * Name of class containing Log instance.
     */
    private String className = null;

    /**
     * Whether to use colorified output. Useful when logging to terminal, less
     * useful when log is redirected to a file.
     */
    private boolean useColors;

    /**
     * Constructor.
     *
     * @param className name of class containing Log instance.
     * @param useColors whether to use colorified output.
     */
    public Log(String className, boolean useColors)
    {
        this.className = className;
        this.useColors = useColors;
    }

    public void log(String str)
    {
        this.log(this.className, str);
    }

    public void log(String className, String str)
    {
        StringBuffer spaces = new StringBuffer();
        for (int i = 0; i < this.indent; i++)
        {
            spaces.append(' ');
        }

        StringBuffer output = new StringBuffer();
        if (useColors)
        {
            str = "$GREEN$GfxTest  $LIGHT_BLUE$" + this.getDate() + "  $ORANGE$" + String.format("%-40s", className)
                    + spaces + "  $GRAY$" + str;
        }
        else
        {
            str = "GfxTest  " + this.getDate() + "  " +  String.format("%-40s", className) + spaces + "  " + str;
        }

        String[] pieces = str.split("\\$");

        for (String piece : pieces)
        {
            if (colors.containsKey(piece.toUpperCase()))
            {
                if (useColors)
                {
                    output.append("\033[0;" + colors.get(piece.toUpperCase()) + "m");
                }
            }
            else
            {
                output.append(piece);
            }
        }
        // vypis vytvoreneho retezce na standardni vystup
        if (useColors)
        {
            System.out.println(output.append("\033[0m"));
        }
        else
        {
            System.out.println(output);
        }
    }

    public void logSet(String str, Object o)
    {
        String value = o == null ? "$RED$null" : "$GRAY$" + o.toString();
        this.log("$LIGHT_BLUE$" + str + "$ORANGE$ = " + value);
    }

    public void logBegin(String className, String str)
    {
        this.log(className, str + " $GREEN$begin");
        this.indent += this.indentDelta;
    }

    public void logBegin(String str)
    {
        this.logBegin(this.className, str);
    }

    public void logEnd(String className, String str)
    {
        if (this.indent >= this.indentDelta)
        {
            this.indent -= this.indentDelta;
        }
        this.log(className, str + " $VIOLET$end");
    }

    public void logEnd(String str)
    {
        this.logEnd(this.className, str);
    }

    public void logError(String str)
    {
        this.logError(this.className, str);
    }

    public void logError(String className, String str)
    {
        this.log(className, "$RED$error $GRAY$" + str);
    }

    public static void main(String[] args)
    {
        System.out.println("test start");
        System.out.println("test end");
    }

    @SuppressWarnings("deprecation")
    private String getDate()
    {
        // vytvorime pomocny buffer, do ktereho seskladame retezec
        Date date = new Date();
        return String.format("%04d-%02d-%02d  %02d:%02d:%02d", 1900 + date.getYear(), date.getMonth(), date.getDay(),
                date.getHours(), date.getMinutes(), date.getSeconds());
    }

}
